import 'dart:async';

import 'package:flutter/material.dart';
import 'package:global/global.dart';

class GSearchableTextFieldDialog extends StatefulWidget {
  final String hintText;
  final Future<Widget?> Function(String text) onSearch;
  final TextStyle? style, hintStyle;
  final Color? progressColor;

  const GSearchableTextFieldDialog({
    required this.hintText,
    required this.onSearch,
    this.style,
    this.hintStyle,
    this.progressColor,
    super.key,
  });

  @override
  GSearchableTextFieldDialogState createState() =>
      GSearchableTextFieldDialogState();
}

class GSearchableTextFieldDialogState
    extends State<GSearchableTextFieldDialog> {
  final TextEditingController _dialogController = TextEditingController();
  Timer? _searchTimer;
  Widget? _resultWidget;
  bool _isLoading = false;
  final FocusNode _focusNode = FocusNode();

  @override
  void initState() {
    super.initState();
    Future.delayed(
        const Duration(milliseconds: 50), () => _focusNode.requestFocus());
  }

  @override
  Widget build(BuildContext context) => Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          GUtils.getTextFormField(
            controller: _dialogController,
            focusNode: _focusNode,
            style: widget.style ?? Global.defTFFEnabledStyle,
            hintText: widget.hintText,
            labelStyle: widget.hintStyle ?? Global.defTFFHintOrLabelStyle,
            floatingLabelStyle: Global.defTFFFloatingLabelStyle,
            suffixIcon: Visibility(
              visible: _dialogController.text.isNotEmpty,
              child: IconButton(
                icon: const Icon(
                  Icons.close_sharp,
                  color: Color(0xFF4D4D4D),
                ),
                onPressed: () {
                  _focusNode.requestFocus();
                  setState(() {
                    _isLoading = false;
                    _resultWidget = null;
                    _dialogController.text = '';
                  });
                },
              ),
            ),
            contentPadding: const EdgeInsets.all(10),
            focusedBorder: GUtils.getRoundOutlineInputBorder(width: 0),
            border: GUtils.getRoundOutlineInputBorder(width: 0),
            onChanged: (text) {
              setState(() {
                _isLoading = false;
                _resultWidget = null;
              });
              _searchTimer?.cancel();
              if (text.length > 2) {
                _searchTimer = Timer(
                  const Duration(milliseconds: 1000),
                  () {
                    setState(() => _isLoading = true);
                    widget.onSearch(text).then((widget) {
                      setState(() {
                        _resultWidget = _dialogController.text.trim().isEmpty ||
                                _dialogController.text.trim().length < 3
                            ? null
                            : widget;
                        _isLoading = false;
                      });
                    });
                  },
                );
              }
            },
          ),
          Builder(
              builder: (context) => _isLoading
                  ? Container(
                      decoration: GUtils.getRoundBoxDecoration(
                          radius: 10, borderWidth: 0),
                      height: 70,
                      child: Align(
                        alignment: Alignment.center,
                        child: CircularProgressIndicator(
                            color: widget.progressColor ?? Global.themeColor),
                      ),
                    )
                  : _resultWidget == null
                      ? const SizedBox()
                      : _resultWidget!)
        ],
      );

  static Widget getNoResultsFoundWidget({required TextStyle messageStyle}) =>
      Container(
        width: double.infinity,
        padding: const EdgeInsets.all(10),
        decoration: GUtils.getRoundBoxDecoration(borderWidth: 0),
        child: Text(
          GTexts.noResultsFound,
          textAlign: TextAlign.center,
          style: messageStyle,
        ),
      );

  static ListView getResultsList({
    required List<dynamic> resultList,
    required Widget Function(dynamic resultItem) getResultItemWidget,
    Color? resultItemOverlayColor,
  }) =>
      ListView.separated(
        shrinkWrap: true,
        padding: EdgeInsets.zero,
        itemCount: resultList.length,
        itemBuilder: (context, index) => TextButton(
            style: ButtonStyle(
              alignment: Alignment.centerLeft,
              overlayColor: MaterialStateColor.resolveWith((states) =>
                  resultItemOverlayColor ??
                  Global.themeColor.withOpacity(0.25)),
              padding: MaterialStateProperty.all(const EdgeInsets.all(10)),
              tapTargetSize: MaterialTapTargetSize.shrinkWrap,
            ),
            onPressed: () => Navigator.pop(context, resultList[index]),
            child: getResultItemWidget(resultList[index])),
        separatorBuilder: (context, index) => Container(
          margin: const EdgeInsets.only(left: 10, right: 10),
          width: double.infinity,
          height: 0.5,
          color: const Color(0xFFD4D4D4),
        ),
      );

  static Widget getResultContainer(Widget resultWidget) => ConstrainedBox(
        constraints: const BoxConstraints(minHeight: 30, maxHeight: 250),
        child: Container(
          decoration: GUtils.getRoundBoxDecoration(radius: 10, borderWidth: 0),
          child: resultWidget,
        ),
      );
}
